-- dff
--
-- dff modelliert ein D-FlipFlop mit
-- Freigabeeingang und asynchronem Setz- und Ruecksetzeingang
-- (schaltet bei positiver Taktflanke)

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;

entity dff is
	port( d		: in	std_logic;	-- Eingang
			clk	: in	std_logic;	-- Takt
			ena	: in	std_logic := '1';	-- Freigabe-Eingang
			pr_n	: in	std_logic := '1';	-- asynchroner Setzeingang
			clr_n	: in	std_logic := '1';	-- asynchroner Setzeingang
			q		: out	std_logic; -- Ausgang
			q_n	: out	std_logic);	-- negierter Ausgang
end entity dff;


architecture structure1 of dff is
begin
	process(d, clk, ena, pr_n, clr_n) is
		variable FlipFlop : std_logic; -- Speicherinhalt
	begin

		assert not (pr_n='0' and clr_n='0') report "Sie haben versucht, das Register geleichzeitig zu setzten (pr_n='0') und rueckzusetzen (clr_n='0') !" severity warning;

		if clr_n='0' then
			FlipFlop := '0';
		elsif pr_n='0' then
			FlipFlop := '1';
		elsif clk='1' and clk'event and ena='1' then
			FlipFlop := d;
		end if; 	
		q <= FlipFlop;
		q_n <= not FlipFlop;
	end process;
end architecture structure1;
