-- fifo_ctrl
-- Steuerung von fifo
-- Uebung 4 Entwurf digitaler Systeme

library ieee;
use ieee.std_logic_1164.all;


entity fifo_ctrl is
	generic(width : natural := 8; max_items : positive := 3);
	port(
		clk : in std_logic; 
		clr_n : in std_logic;
		wr_ena : in std_logic;
		rd_ena : in std_logic;
		full : out std_logic;
		empty : out std_logic;
		sel : out std_logic_vector(max_items downto 1);
		ena : out std_logic_vector(max_items downto 1)
	);
end fifo_ctrl;


architecture behave of fifo_ctrl is
		signal wr_pnt : natural := 0;
begin

	process (clk)
		subtype t_01 is natural range 0 to 1; 
		variable write : t_01; -- # geschriebener Daten
		variable read : t_01; -- # gelesener Daten
	begin
		if clr_n ='0' or clr_n='L' then 
			wr_pnt <= 0;
		else
			if (clk = '1' or clk = 'H') and clk'event then
				case wr_ena is
					when '1'|'H' => write := 1;
					when others => write := 0;
				end case;
				case rd_ena is
					when '1'|'H' => read := 1;
					when others => read := 0;
				end case;
				if wr_pnt = max_items and write=1 and read=0 then
					report "FIFO: In die FIFO kann kein Datum geschrieben werden, da sie voll ist !" severity error;
				elsif wr_pnt = 0 and write=0 and read=1 then
					report "FIFO: Aus der FIFO kann kein Datum gelesen werden, da sie leer ist !" severity error;
				else
					wr_pnt <= wr_pnt + write - read;
				end if;
			end if;
		end if;
	end process;

	process (wr_pnt,wr_ena,rd_ena,clr_n)
	begin
			for i in max_items downto 1 loop
				if i < wr_pnt then 
					sel(i) <= '1';
				else
					sel(i) <= '0';
				end if;
				if i > wr_pnt then 
					ena(i) <= '1';
				else
					ena(i) <= rd_ena;
				end if;
			end loop;
	end process;

	full <= '1' when wr_pnt = max_items else '0';
	empty <= '1' when wr_pnt = 0 else '0';
	
end architecture behave;
