-- interface
-- 
-- interface modelliert eine vereinfachte Tastaturschnittstelle 
-- aus Uebung 3, Aufgabe 3 (Entwurf digitaler Systeme)
-- ueberarbeitet fuer Uebung 4
-- Randbedingung: Takt clk muss mit deutlich hoeherer Frequenz als Tastaturtakt tclk betrieben werden


library ieee;
use ieee.std_logic_1164.all;
use work.all;

entity interface is
	generic(WIDTH : positive := 8); -- WIDTH = Laenge des Scan-Codes einer Taste
	port(
		clk :in std_logic; -- Takteingang
		clr_n : in std_logic := '1'; -- asynchroner Ruecksetzeingang
		tclk :in std_logic; -- Tastatur-Takteingang
		tdata : in std_logic; -- serieller Dateneingang fuer die Tastatur
		rd_ena : in std_logic; -- Daten an data_out wurden ausgelesen
		data_valid : out std_logic; -- Daten an data_out sind gueltig
		data_out : out std_logic_vector(WIDTH-1 downto 0); -- paralleler Datenausgang (WIDTH Datenbits)
		parity_err : out std_logic -- '1' = Parity-Fehler
	);
end entity interface;

architecture struct1 of interface is
	signal sreg_data_out : std_logic_vector(WIDTH+2 downto 0); 
	signal ready, ready_sync : std_logic; -- Tastatursignal ist vollstaendig im Schieberegister
	signal reg_ena : std_logic; -- Speicherregister kann Daten uebernehmen
	signal ena_data_valid : std_logic; -- Gueltigkeit der Daten uebernehmen
	signal sreg_clr_n, sreg_clr : std_logic; -- Reset des Schieberegisters
	signal tdata_n : std_logic; -- not tdata
	signal parity_next, parity_wrong : std_logic; -- Parity-Pruefung
begin

	tdata_n <= not tdata;

	Schieberegister: entity sreg 
		generic map(WIDTH => WIDTH+3)
		port map(
			clk => tclk,
			clr_n => sreg_clr_n,
			line_in => tdata_n,
			parallel_out_n => sreg_data_out,
			parallel_out(0) => ready
		);
		
	Speicherregister: entity reg 
		generic map(WIDTH => WIDTH)
		port map(
			d => sreg_data_out(WIDTH downto 1), 
			clk => clk,
			q => data_out,
			ena => reg_ena,
			clr_n => clr_n
		);
		
	Synchronisationsflipflop: entity dff
		port map(
			d => ready,
			clk => clk,
			q => ready_sync,
			clr_n => clr_n
		);
		
	Flankendetektor: entity edge 
		port map(
			clk => clk,
			clr_n => clr_n,
			data => ready_sync,
			detected => reg_ena
		);

	Verzoegerungsflipflop: entity dff
		port map(
			d => reg_ena,
			clk => clk,
			q => sreg_clr,
			pr_n => clr_n
		);

	sreg_clr_n <= not sreg_clr;

	data_valid_Flipflop: entity dff
		port map(
			d =>  reg_ena,
			ena => ena_data_valid,
			clk => clk,
			q => data_valid,
			clr_n => clr_n
		);
		
	ena_data_valid <= reg_ena or rd_ena;

	-- Paritaetspruefung:
	calculate_parity_flipflop: entity dff
		port map(
			d => parity_next,
			clk => tclk,
			q => parity_wrong,
			clr_n => sreg_clr_n  
		);
		
	parity_err_flipflop: entity dff
		port map(
			d => parity_wrong,
			clk => clk,
			ena => reg_ena,
			q => parity_err,
			clr_n => clr_n  
		);

	parity_next <= tdata xor parity_wrong;

	Ueberpruefung_Stopbit: process (reg_ena, sreg_data_out(WIDTH+2))
	begin
		assert not (reg_ena = '1' and sreg_data_out(WIDTH+2) /= '1') report "Tastaturinterface: Der Tastaturcode war nicht mit einem Stopbit (='1') abgeschlossen  !" severity error;
	end process Ueberpruefung_Stopbit;		

	Ueberpruefung_Paritybit: process (ready_sync, parity_wrong)
	begin
		assert not (parity_wrong='1' and ready_sync='1') report "Tastaturinterface: Das Paritaetsbit der Tastatur war falsch gesetzt !" severity note;
	end process Ueberpruefung_Paritybit;		
	
end architecture struct1;

