-- read
-- liest Daten aus dem Tastaturinterface
-- (fr Testbench)

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;

entity read is
	generic(width : natural := 8); -- Breite des Dateneingangs tcode
	port(
		tcode : in std_logic_vector(width-1 downto 0); -- Dateineingang
		data_valid : in std_logic; 
		parity_error : in std_logic; -- : Paritaetsfehler 
		rd_ena : out std_logic := '0' -- Daten von tcode wurden ausausgelesen 
	);
end entity read;

architecture behave of read is

	function to_string(v : std_logic_vector) return string is
		variable st : string(v'range);
	begin 
		for i in v'range loop
			case v(i) is
				when '0' =>  st(i) := '0';
				when '1' =>  st(i) := '1';
				when 'L' =>  st(i) := 'L';
				when 'H' =>  st(i) := 'H';
				when 'U' =>  st(i) := 'U';
				when 'X' =>  st(i) := 'X';
				when 'W' =>  st(i) := 'W';
				when 'Z' =>  st(i) := 'Z';
				when '-' =>  st(i) := '-';
			end case;
		end loop;
		return st;
	end to_string;

begin

	lesen: process (data_valid)
	begin
		if (data_valid = '1' or data_valid ='H') and data_valid'event then -- gueltiger Tastaturcode lieg an
			report to_string(tcode) severity note; -- Tastaturcode ausgeben
			rd_ena <= '1'; -- wurde erfolgreich gelesen 
		else
			rd_ena <= '0'; 
		end if;
	end process lesen;
	
	Ueberpruefung: process (parity_error)
	begin
		assert parity_error /= '1' and parity_error /= 'H' report "Read: Parity-Error" severity note;
	end process Ueberpruefung;
	
end architecture behave;
