-- sreg
--
-- sreg modelliert ein Schieberegister der Breite WIDTH aus 
-- D-Flip-Flops mit seriellem Eingang, parallelem Ausgang (auch negiert)
-- und asynchronem Setz- und Ruecksetzeingang (negiert)

library ieee;
use ieee.std_logic_1164.all;
use work.all;

entity sreg is
	generic(WIDTH: positive := 2);
	port(
		clk :in std_logic; -- Takteingang
		line_in : in std_logic; -- serieller Dateneingang
		ena : in std_logic := '1'; -- Enable fuer den Takt
		clr_n : in std_logic := '1'; -- asynchroner Ruecksetzeingang (negiert)
		pr_n : in std_logic := '1'; -- asynchroner Setzeingang (negiert)
		parallel_out : out std_logic_vector(WIDTH - 1 downto 0); -- paralleler Datenausgang
		parallel_out_n : out std_logic_vector(WIDTH - 1 downto 0) -- negierter paralleler Datenausgang
	);
end entity sreg;


architecture struct of sreg is
	signal wire : std_logic_vector(WIDTH downto 0);
	signal wire_n : std_logic_vector(WIDTH downto 0);
begin
	chain: for i in WIDTH-1 downto 0 generate
	begin
		d_ff: entity dff
			port map(
				ena => ena,
				clk => clk,
				clr_n => clr_n,
				pr_n => pr_n,
				d => wire(i+1),
				q => wire(i),
				q_n => wire_n(i)
			);
	end generate chain;
	wire(WIDTH) <= line_in;
	parallel_out <= wire(WIDTH-1 downto 0);
	parallel_out_n <= wire_n(WIDTH-1 downto 0);
end architecture struct;


